/*
 * Rahmenwerk-Plug-in "Anlagenstatus"
 * Copyright (C) 2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.anlagenstatus.figures;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.GridData;
import org.eclipse.draw2d.GridLayout;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.LineBorder;
import org.eclipse.draw2d.Panel;
import org.eclipse.draw2d.ScalableFigure;
import org.eclipse.draw2d.XYLayout;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.swt.graphics.Font;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.plugin.anlagenstatus.einstellungen.PluginEinstellungen;
import de.bsvrz.buv.plugin.dobj.DoFigure;
import de.bsvrz.buv.plugin.dobj.decorator.ZoomVerhaltenFixFigure;

/**
 * Figure zur Anzeige der Statusinformationen.
 * 
 * @author BitCtrl Systems GmbH, Steffen Gieseler
 * @version $Id: AnlagenFigure.java 46802 2013-10-23 15:13:00Z gieseler $
 */
public class AnlagenFigure extends Panel implements ScalableFigure, DoFigure,
		ZoomVerhaltenFixFigure {

	/** Scale. */
	private double scale = 1.0;

	/** Zoom. */
	private double zoom = 1.0;

	/** Dimension. */
	private Dimension dimension;

	/** Datapanel. */
	private final Panel dataPanel;

	/** Flag ZoomFix. */
	private boolean zoomFix;

	/** Mindest-Rahmenbreite bei der Anzeige. */
	private static final int MIN_BORDERWIDTH = 3;

	/**
	 * Konstruktor.
	 */
	public AnlagenFigure() {
		super();
		setLayoutManager(new XYLayout());
		setBorder(new LineBorder(ColorConstants.gray, PluginEinstellungen
				.getInstance().getEinstellungen().getRahmenBreite()));

		dataPanel = new Panel();
		dataPanel.setLayoutManager(new GridLayout(1, false));
		final Font defaultFont = PlatformUI.getWorkbench()
				.getActiveWorkbenchWindow().getWorkbench().getDisplay()
				.getSystemFont();
		dataPanel.setFont(defaultFont);
		add(dataPanel);
	}

	/**
	 * Setzt die Gr&ouml;sse der Figur.
	 * 
	 * @param dimension
	 *            Gr&ouml;sse der Figur
	 */
	public void setDimension(Dimension dimension) {
		this.dimension = dimension;
	}

	/**
	 * Gibt das Datenpanel zur&uuml;ck.
	 * 
	 * @return Datenpanel
	 */
	public Panel getDataPanel() {
		return dataPanel;
	}

	/**
	 * F&uuml;gt eine Figure hinzu.
	 * 
	 * @param figure
	 *            Figure
	 * @param constraint
	 *            Constraint
	 */
	public final void addToFigure(final IFigure figure, final Object constraint) {
		dataPanel.add(figure, constraint);

		final Dimension preferredSize = dataPanel.getPreferredSize();

		dataPanel.setSize(preferredSize);

		getLayoutManager().setConstraint(dataPanel,
				new Rectangle(0, 0, preferredSize.width, preferredSize.height));

		setSize(getPreferredSize());
		dimension = getSize();
	}

	/**
	 * F&uuml;gt einen Label hinzu.
	 * 
	 * @param label
	 *            Label
	 */
	public final void addDataLabel(final Label label) {

		final GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.CENTER;
		gridData.verticalAlignment = GridData.CENTER;
		gridData.grabExcessHorizontalSpace = true;

		addToFigure(label, gridData);
	}

	/** {@inheritDoc} */
	@Override
	public double getScale() {
		return scale;
	}

	/** {@inheritDoc} */
	@Override
	public void setScale(final double scale) {
		if (this.scale == scale) {
			return;
		}

		this.scale = scale;

		updateFigure();

		for (Object o : getChildren()) {
			if (o instanceof ScalableFigure) {
				((ScalableFigure) o).setScale(scale);
			} else {
				if (o instanceof Panel) {
					for (Object oo : ((IFigure) o).getChildren()) {
						if (oo instanceof ScalableFigure) {
							((ScalableFigure) oo).setScale(scale);
						}
					}
				}
			}
		}

	}

	/**
	 * Gibt den aktuellen ZOOM-Level zur&uuml;ck.
	 * 
	 * @return aktueller ZOOM-Level
	 */
	public double getZoom() {
		return zoom;
	}

	/**
	 * Aktualisiert die Figure.
	 */
	public void updateFigure() {
		if (dimension == null) {
			dimension = getSize();
		}

		int width = dimension.width;
		int height = dimension.height;
		final double combinedScale = zoom / scale;

		if (isZoomVerhaltenFix()) {
			width = (int) (dimension.width / combinedScale);
			if ((width % 2) != 0) {
				width++;
			}
			height = (int) (dimension.height / combinedScale);
			if ((height % 2) != 0) {
				height++;
			}
		} else {
			width = (int) (dimension.width * scale);
			if ((width % 2) != 0) {
				width++;
			}
			height = (int) (dimension.height * scale);
			if ((height % 2) != 0) {
				height++;
			}
		}

		setSize(width, height);

		((LineBorder) getBorder()).setWidth(calcBorderWidth());

		repaint();
	}

	/**
	 * Berechnet die Rahmenbreite mit einem Minimum.
	 * 
	 * @return Rahmenbreite
	 */
	private int calcBorderWidth() {
		final double amount;
		if (isZoomVerhaltenFix()) {
			amount = 1 / zoom / scale;
		} else {
			amount = scale;
		}

		// final double combinedScale = zoom / scale;
		// int borderwidth = (int) (PluginEinstellungen.getInstance()
		// .getEinstellungen().getRahmenBreite() / combinedScale);

		int borderwidth = (int) (PluginEinstellungen.getInstance()
				.getEinstellungen().getRahmenBreite() * amount);

		if ((borderwidth * zoom) < MIN_BORDERWIDTH) {
			borderwidth = (int) (MIN_BORDERWIDTH / zoom);
		}

		return borderwidth;
	}

	/** {@inheritDoc} */
	@Override
	protected final void paintChildren(final Graphics graphics) {
		final double amount;
		if (isZoomVerhaltenFix()) {
			amount = 1 / zoom / scale;
		} else {
			amount = scale;
		}

		if (amount != 1.0) {
			graphics.scale(amount);
			graphics.pushState();
			super.paintChildren(graphics);
			graphics.popState();
		} else {
			super.paintChildren(graphics);
		}
	}

	/** {@inheritDoc} */
	@Override
	protected final boolean useLocalCoordinates() {
		return true;
	}

	/** {@inheritDoc} */
	@Override
	public void handleZoomChanged(final double newZoom) {
		if (zoom == newZoom) {
			return;
		}

		zoom = newZoom;

		updateFigure();

		// if (isZoomVerhaltenFix()) {
		for (Object o : getChildren()) {
			if (o instanceof DoFigure) {
				((DoFigure) o).handleZoomChanged(zoom);
			} else {
				if (o instanceof Panel) {
					for (Object oo : ((IFigure) o).getChildren()) {
						if (oo instanceof DoFigure) {
							((DoFigure) oo).handleZoomChanged(zoom);
						}
					}
				}
			}
		}
		// }
	}

	/** {@inheritDoc} */
	@Override
	public Point getHotspot() {
		return getLocation();
	}

	/** {@inheritDoc} */
	@Override
	public void setHotspot(final Point hotspot) {
		setLocation(hotspot);
	}

	/** {@inheritDoc} */
	@Override
	public void setSichtbareZoomStufe(final double vonZoom, final double bisZoom) {
	}

	/** {@inheritDoc} */
	@Override
	public boolean isZoomVerhaltenFix() {
		return zoomFix;
	}

	/** {@inheritDoc} */
	@Override
	public void setZoomVerhaltenFix(boolean zoomVerhaltenFix) {
		zoomFix = zoomVerhaltenFix;
	}
}
