/*
 * Rahmenwerk-Plug-in "Anlagenstatus"
 * Copyright (C) 2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.anlagenstatus.modell;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import de.bsvrz.buv.plugin.anlagenstatus.einstellungen.PluginEinstellungen;
import de.bsvrz.buv.plugin.anlagenstatus.impl.AnlagenstatusDoModelImpl;
import de.bsvrz.buv.plugin.anlagenstatus.internal.PluginAnlagenStatus;
import de.bsvrz.sys.funclib.bitctrl.modell.ObjektFactory;
import de.bsvrz.sys.funclib.bitctrl.modell.SystemObjekt;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.objekte.Uz;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.objekte.Viz;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.objekte.Vrz;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Modell f&uuml;r den Anlagenstatus.
 * 
 * @author BitCtrl Systems GmbH, Steffen Gieseler
 * @version $Id: AnlagenStatusModell.java 45198 2013-07-25 08:49:37Z gieseler $
 */
public class AnlagenStatusModell extends AnlagenstatusDoModelImpl {

	/** Hierarchielevel VRZ/VIZ. */
	public static final int VRZ_VIZ_LEVEL = 0;

	/**
	 * Map zur Zuordnung von Systemobjekten zu Modell-Knoten.
	 */
	private final Map<SystemObjekt, AnlagenStatusKnoten> systemObjekt2NodeMap = new HashMap<SystemObjekt, AnlagenStatusKnoten>();

	/**
	 * Liste der Knoten, die f&uuml;r den View initailisiert werden sollen.
	 * <p>
	 * (Enth&auml;lt nur die Ger&auml;te und Steuermodule. Die Kinder der
	 * Steuermodule werden von diesen selbst initialisiert.)
	 * </p>
	 */
	private final List<AnlagenStatusKnoten> viewKnoten = new ArrayList<AnlagenStatusKnoten>();

	/**
	 * Debug-Logger.
	 */
	private final Debug debug = Debug.getLogger();

	/** Flag, ob die Ausrichtung TOP/DOWN (true) ist oder LEFT/RIGHT (false). */
	private boolean topdown = true;

	/**
	 * Flag, ob das Routing mit Manhattan-Algorithmus erfolgen soll (true), oder
	 * mit FAN-Routing (false).
	 */
	private boolean manhattanrouting = true;

	/** aktuell angezeigte Ebene. */
	private int showLevel = VRZ_VIZ_LEVEL;

	/** Anzahl der Ebenen. */
	private int maxLevel = VRZ_VIZ_LEVEL;

	/**
	 * Gibt die Anzahl der Ebenen zur&uuml;ck.
	 * 
	 * @return Anzahl der Ebenen
	 */
	public int getMaxLevel() {
		return maxLevel;
	}

	/**
	 * Maximale Anzahl von Verbindungen eines Knotens im Modell.
	 */
	private int verbindungenMax;

	/**
	 * Konstruktor.
	 */
	public AnlagenStatusModell() {
		debug.config("Anlagenstatus-Plugin: Das Modell wird initialisiert");

		final ObjektFactory objektFactory = PluginAnlagenStatus.getDefault()
				.getObjektFactory();

		List<? extends SystemObjekt> objekte = objektFactory
				.bestimmeModellobjekte(Vrz.PID);

		if (objekte.size() == 0) {
			// keine VRZ gefunden, wir beginnen mit UZ
			objekte = objektFactory.bestimmeModellobjekte(Uz.PID);
		}

		for (SystemObjekt s : objekte) {
			new AnlagenStatusGeraetKnoten(this, null, s, VRZ_VIZ_LEVEL);
		}

		for (SystemObjekt s : objektFactory.bestimmeModellobjekte(Viz.PID)) {
			new AnlagenStatusGeraetKnoten(this, null, s, VRZ_VIZ_LEVEL);
		}
	}

	/**
	 * Gibt die aktuell angezeigte Ebene zur&uuml;ck.
	 * 
	 * @return Level der aktuell angezeigteh Ebene
	 */
	public int getShowLevel() {
		return showLevel;
	}

	/**
	 * Aktualisiert die aktuell angezeigte Ebene.
	 */
	public void updateShowLevel() {
		this.showLevel = VRZ_VIZ_LEVEL;
		for (AnlagenStatusKnoten knoten : getViewNodes()) {
			if (knoten.isSichtbar() && knoten.getlevel() > showLevel) {
				showLevel = Math.max(showLevel, knoten.getlevel());
			}
		}
	}

	/**
	 * Setzt die aktuell angezeigte Ebene.
	 * 
	 * @param showLevel
	 *            aktuell angezeigte Ebene
	 */
	public void setShowLevel(final int showLevel) {
		this.showLevel = showLevel;

		for (AnlagenStatusKnoten knoten : getViewNodes()) {
			if (knoten.getlevel() > showLevel) {
				knoten.ausblenden();
			} else {
				knoten.einblenden();
			}
		}
		this.showLevel = showLevel;
	}

	/**
	 * Gibt an, ob das Routing mit Manhattan-Algorithmus erfolgen soll.
	 * 
	 * @return true, wenn Manhattan-Algorithmus, sonst false.
	 */
	public boolean isManhattanrouting() {
		return manhattanrouting;
	}

	/**
	 * Setzt das Routing.
	 * 
	 * @param manhattanrouting
	 *            Routing mit Manhattan-Algorithmus ja (true) oder nein (false)
	 */
	public void setManhattanrouting(final boolean manhattanrouting) {
		this.manhattanrouting = manhattanrouting;
	}

	/**
	 * Gibt an, ob Ausrichtung TOP/DOWN erfolgen soll.
	 * 
	 * @return true, wenn Ausrichtung TOP/DOWN, sonst false.
	 */
	public boolean isTopDown() {
		return topdown;
	}

	/**
	 * Setzt die Ausrichtung.
	 * 
	 * @param istopdown
	 *            true, wenn Ausrichtung TOP/DOWN, sonst false
	 */
	public void setTopDown(final boolean istopdown) {
		this.topdown = istopdown;
	}

	/**
	 * Gibt die Liste der im View zu initialisierenden Knoten zur&uuml;ck.
	 * 
	 * @return Liste von Knoten
	 */
	public List<AnlagenStatusKnoten> getViewNodes() {
		return viewKnoten;
	}

	/**
	 * F&uuml;gt einen Knoten zum Modell hinzu.
	 * 
	 * @param node
	 *            Knoten
	 */
	public void addNode(final AnlagenStatusKnoten node) {
		systemObjekt2NodeMap.put(node.getSystemObjekt(), node);
		if (node instanceof AnlagenStatusGeraetKnoten
				|| node instanceof SteuerModulKnoten) {
			viewKnoten.add(node);
			maxLevel = Math.max(maxLevel, node.getlevel());

			if (node instanceof SteuerModulKnoten
					&& !PluginEinstellungen.getInstance().getEinstellungen()
							.isSteuerModuleAusblenden()
					|| node instanceof AnlagenStatusGeraetKnoten) {
				showLevel = Math.max(showLevel, node.getlevel());
			}

		}
		verbindungenMax = Math.max(verbindungenMax, node.getSourceConnections()
				.size());
	}

	/**
	 * Gibt den zu einem Systemobjekt zugeh&ouml;rigen Knoten des Modells
	 * zur&uuml;ck.
	 * 
	 * @param systemObjekt
	 *            Systemobjekt
	 * @return Knoten oder null, wenn kein zugeh&ouml;riger Knoten gefunden
	 *         wurde
	 */
	public AnlagenStatusKnoten getKnoten(final SystemObjekt systemObjekt) {
		return systemObjekt2NodeMap.get(systemObjekt);
	}

	/**
	 * Gibt die maximale Anzahl von Verbindungen eines Knotens im Modell
	 * zur&uuml;ck.
	 * 
	 * @return maximale Anzahl von Verbindungen eines Knotens
	 */
	public int getVerbindungenMax() {
		return verbindungenMax;
	}
}
